/*
   rpcclnt.c -- PIKT rpc client (piktc) routines

   $Header: /home/cvs/ucgsb/pikt/src/piktc/rpcclnt.c,v 1.12 1999/05/09 18:39:49 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <rpc/rpc.h>

#if defined(HAVE_NETCONFIG_H)
#include <netconfig.h>
#endif

#include <rpc/xdr.h>
#include <rpc/clnt.h>
#include "rpc.h"

#include <sys/types.h>
#include <sys/stat.h>

#include "common.h"

#include <errno.h>
CLIENT *clnt;

fil _file;
nam alert;

int disablealert(char *, char *, const char *, int);
int enablealert(char *, char *, char *, int);
int installfile(char *, char *, char *, const char *, char *, char *, char *, int);
int deletefile(char *, char *, char *, char *, int);
int fileexist(char *, char *, char *, const char *, int, int);
int showstatus(char *, char *, const char *, int);

/* enable all alerts in the supplied list on the given host
 */
void
enablealerts (char *host, nam *alts, int inform)
{
	nam *np;
	int res;

	if (inform)
		printf("enabling alert(s)...\n");
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform)
				printf("%s not enabled, unknown alert\n", np->name);
			continue;
		}
		if (! (res = fileexist(host, np->name, altdir, ".alt", IGNORE, inform)) && (res != IGNORE)) {
			if (inform)
				printf("%s not enabled, %s.alt not found\n", np->name, np->name);
			continue;
		}
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert(s) not enabled\n");
			return;
		}
		res = enablealert(host, np->name, np->aux, inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert(s) not enabled\n");
			return;
		}
		if (inform)
			printf("%s %senabled\n", np->name, res == 1 ? "" : "not ");
	}
	return;
}

/* disable all alerts in the supplied list on the given host
 */
void
disablealerts (char *host, nam *alts, int inform)
{
	nam *np;
	int res;

	if (inform)
		printf("disabling alert(s)...\n");
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform)
				printf("%s not enabled, unknown alert\n", np->name);
			continue;
		}
		res = disablealert(host, np->name, "", inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert(s) not disabled\n");
			return;
		}
		if (inform)
			printf("%s %sdisabled\n", np->name, res == 1 ? "" : "not ");
	}
	return;
}

/* install all files in the supplied list on the given host
 */
void
installfiles (char *host, nam *fils, char *dir, const char *ext, char *mode, char *uid, char *gid, int inform)
{
	nam *np;
	int res;

	if (inform)
		printf("installing file(s)...\n");
	foreach (np, fils) {
		res = installfile(host, np->name, dir, ext, mode, uid, gid, inform);
		switch (res) {
			case NONE:
			case UNAUTHORIZED:
				if (inform)
					printf("file(s) not installed\n");
				return;
				break;
			case IGNORE:
				continue;
				break;
			default:
				if (inform)
					printf("%s%s %sinstalled\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
				break;
		}
	}
	return;
}

/* delete all files in the supplied list on the given host
 */
void
deletefiles (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int res;

	if (inform)
		printf("deleting file(s)...\n");
	foreach (np, fils) {
		res = deletefile(host, np->name, dir, ext, inform);
		switch (res) {
			case NONE:
			case UNAUTHORIZED:
				if (inform)
					printf("file(s) not deleted\n");
				return;
				break;
			case IGNORE:
				continue;
				break;
			default:
				if (inform)
					printf("%s%s %sdeleted\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
				break;
		}
	}
	return;
}

/* list all files in the supplied list on the given host
 */
void
listfiles (char *host, nam *fils, char *dir, const char *ext, int inform)
{
	nam *np;
	int res;

	if (inform)
		printf("listing file(s)...\n");
	foreach (np, fils) {
		res = fileexist(host, np->name, dir, ext, NIL, inform);
		switch (res) {
			case NONE:
			case UNAUTHORIZED:
				if (inform)
					printf("file(s) not listed\n");
				return;
				break;
			case IGNORE:
				continue;
				break;
			default:
				/* if (inform) */
					printf("%s%s %sfound\n", np->name, strcmp(ext, "NIL") ? ext : "", res == 1 ? "" : "not ");
		}
	}
	return;
}

/* kill either the alert daemon (piktd) or the service daemon (piktc_svc)
   on the given host
 */
void
killdaemon (char *host, unsigned long service, int inform)
{
	int *res;

	res = (int *) NULL;
	if (inform)
		printf("killing daemon (%s)...\n", service == KILL_PIKTD_SERVICE ? "piktd" : "piktc_svc");
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("daemon not killed\n");
		}
		return;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "killdaemon (%s) request to host %s", host, service == KILL_PIKTD_SERVICE ? "piktd" : "piktc_svc");
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (service == KILL_PIKTD_SERVICE)
		res = killpiktd_1((void *) NULL, clnt);
	else
		res = killpiktc_svc_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case ERROR:
					printf("daemon not killed\n");
					break;
				case NONE:
					printf("daemon not running\n");
					break;
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					printf("daemon not killed\n");
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					remove(piktclock);
					exit(1);
					break;
				case 0:
					printf("daemon killed\n");
					break;
				default:
					break;
			}
		} else {
			if (service == KILL_PIKTC_SVC_SERVICE) {
				printf("daemon killed\n");
			} else {
				fprintf(stderr, "no response from server\n");
			}
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	return;
}

/* restart either the alert daemon (piktd) or the service daemon (piktc_svc)
   on the given host
 */
void
restartdaemon (char *host, unsigned long service, int inform)
{
	int *res;

	res = (int *) NULL;
	if (inform)
		printf("(re)starting daemon (%s)...\n", service == RESTART_PIKTD_SERVICE ? "piktd" : "piktc_svc");
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform) {
			clnt_pcreateerror(host);
			printf("daemon not (re)started\n");
		}
		return;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "restartdaemon (%s) request to host %s", host, service == RESTART_PIKTD_SERVICE ? "piktd" : "piktc_svc");
	(AUTH *) clnt->cl_auth = authunix_create_default();
	if (service == RESTART_PIKTD_SERVICE)
		res = restartpiktd_1((void *) NULL, clnt);
	else
		res = restartpiktc_svc_1((void *) NULL, clnt);
	if (inform) {
		if (res) {
			switch (*res) {
				case ERROR:
				case NONE:
					printf("daemon not (re)started\n");
					break;
				case UNAUTHORIZED:
					fprintf(stderr, "service unauthorized\n");
					printf("daemon not (re)started\n");
					break;
				case DENIED:
					fprintf(stderr, "access denied\n");
					remove(piktclock);
					exit(1);
					break;
				case 0:
					printf("daemon (re)started\n");
					break;
				default:
					break;
			}
		} else {
			fprintf(stderr, "no response from server\n");
		}
	}
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	return;
}

/* report the status of each alert in the supplied list on the given host
 */
void
showstata (char *host, nam *alts, int inform)
{
	nam *np;
	int res;

	if (inform)
		printf("showing alert stata...\n");
	foreach (np, alts) {
		if (np->state == ERROR) {
			if (inform)
				printf("%s status not shown, unknown alert\n", np->name);
			continue;
		}
		res = showstatus(host, np->name, "", inform);
		if ((res == NONE) || (res == UNAUTHORIZED)) {
			if (inform)
				printf("alert stata not shown\n");
			return;
		}
		if (inform)
			printf("%s %sactive\n", np->name, res == 1 ? "" : "not ");
	}
	return;
}

/* report the status of the given alert on the given host
 */
int
showstatus (char *host, char *name, const char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = 0;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "showstatus request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = showstatus_1((anam *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* enable the given alert on the given host
 */
int
enablealert (char *host, char *name, char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = ADD;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "editconf request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = editconf_1((anam *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* disable the given alert on the given host
 */
int
disablealert (char *host, char *name, const char *aux, int inform)
{
	int *res;

	res = (int *) NULL;
	alert.name = name;
	alert.aux = aux;
	alert.state = DELETE;
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "editconf request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = editconf_1((anam *) &alert, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* install the given file on the given host
 */
int
installfile (char *host, char *name, char *dir, const char *ext, char *mode, char *uid, char *gid, int inform)
{
	static nam *np;
	static char *fn;
	struct stat fbuf;
	off_t size;
	FILE *fp;
	int *res;

	res = (int *) NULL;
	_file.name = name;
	_file.extension = ext;
	if (!*dir) {	/* signals that this is an other file
			   install, in which case override
			   the default dir, mode, uid & gid
			 */
		if (!(np = lookupname(files, name))) {
			if (!filesall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in installfile(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.directory = np->aux;
		_file.mode = np->mode;
		_file.uid = np->uid;
		_file.gid = np->gid;
	} else if (!strcmp(ext, "NIL")) {   /* signals that this is a program
				               install, in which case possibly
				               override the default dir, mode,
				               uid & gid
				             */
		if (!(np = lookupname(programs, name))) {
			if (!programsall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in installfile(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->aux && *(np->aux))
			_file.directory = np->aux;
		else
			_file.directory = dir;
		_file.mode = np->mode;
		_file.uid = np->uid;
		_file.gid = np->gid;
	} else {			/* go with the defaults */
		_file.directory = dir;
		_file.mode = mode;
		_file.uid = uid;
		_file.gid = gid;
	}
	fn = (char *) malloc((strlen(stgdir)+strlen(name)+strlen(ext)+2) * sizeof(char));
	sprintf(fn, "%s/%s%s", stgdir, name, strcmp(ext, "NIL") ? ext : "");
	stat(fn, &fbuf);
	size = fbuf.st_size;
	_file.size = size;
	_file.text = (char *) malloc((size_t)(size+1)*sizeof(char));
	if (!_file.text)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in installfile(), malloc() failure for _file.text");
	if ((fp = fopen(fn, "r")) == NULL) {
		logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in installfile(), fopen() error for %s", fn);
		free(fn);
		return 0;
	}
	fread(_file.text, sizeof(char), (size_t)size, fp);
	if (fclose(fp)) {
		logreturn(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in installfile(), fclose() error for %s", fn);
		free(fn);
		return 0;
	}
	free(fn);
	*(_file.text+size) = '\0';
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "tcp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "installfile request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = installfile_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	FREE(_file.text);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* delete the given file on the given host
 */
int
deletefile (char *host, char *name, char *dir, char *ext, int inform)
{
	static nam *np;
	int *res;

	res = (int *) NULL;
	_file.name = name;
	_file.extension = ext;
	if (!*dir) {	/* signals that this is an other file
			   deletion, in which case override
			   the default dir
			 */
		if (!(np = lookupname(files, name))) {
			if (!filesall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in deletefile(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.directory = np->aux;
	} else if (!strcmp(ext, "NIL")) {   /* signals that this is a program
				               deletion, in which case possibly
				               override the default dir
				             */
		if (!(np = lookupname(programs, name))) {
			if (!programsall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in deletefile(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->aux && *(np->aux))
			_file.directory = np->aux;
		else
			_file.directory = dir;
	} else {
		_file.directory = dir;
	}
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "deletefile request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = deletefile_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				return UNAUTHORIZED;
				break;
			default:
				return FALSE;
				break;
		}
	} else
		return NONE;
}

/* report if the given file exists on the given host
 */
int
fileexist (char *host, char *name, char *dir, const char *ext, int status, int inform)
{
	static nam *np;
	int *res;

	res = (int *) NULL;
	_file.name = name;
	_file.extension = ext;
	if (!*dir) {	/* signals that this is an other file
			   lookup, in which case override
			   the default dir
			 */
		if (!(np = lookupname(files, name))) {
			if (!filesall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in fileexist(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		_file.directory = np->aux;
	} else if (!strcmp(ext, "NIL")) {   /* signals that this is a program
				               lookup, in which case possibly
				               override the default dir
				             */
		if (!(np = lookupname(programs, name))) {
			if (!programsall) {
				logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in fileexist(), lookup failure for %s", name);
				return 0;
			} else
				return IGNORE;
		}
		if (np->aux && *(np->aux))
			_file.directory = np->aux;
		else
			_file.directory = dir;
	} else {
		_file.directory = dir;
	}
	_file.mode = "";
	_file.uid = "";
	_file.gid = "";
	_file.size = 0;
	_file.text = "";
	clnt = clnt_create(host, PIKTCPROG, PIKTCVERS, "udp");
	if (clnt == NULL) {
		if (inform)
			clnt_pcreateerror(host);
		return NONE;
	}
	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "fileexist request to host %s", host);
	(AUTH *) clnt->cl_auth = authunix_create_default();
	res = fileexist_1((rfil *) &_file, clnt);
	if (res) {
		switch (*res) {
			case UNAUTHORIZED:
				if (status != IGNORE)
					fprintf(stderr, "service unauthorized\n");
				break;
			case DENIED:
				fprintf(stderr, "access denied\n");
				remove(piktclock);
				exit(1);
				break;
			default:
				break;
		}
	} else
		fprintf(stderr, "no response from server\n");
	/* auth_destroy(clnt->cl_auth); */
	clnt_destroy(clnt);
	if (res) {
		switch (*res) {
			case 0:
				return TRUE;
				break;
			case UNAUTHORIZED:
				if (status == IGNORE)
					return IGNORE;
				else
					return UNAUTHORIZED;
				break;
			default:
				if (status == IGNORE)
					return IGNORE;
				else
					return FALSE;
				break;
		}
	} else
		return NONE;
}
