/*
   piktd.c -- PIKT alert daemon

   $Header: /home/cvs/ucgsb/pikt/src/piktd/piktd.c,v 1.9 1999/05/09 18:36:43 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-99 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA

   --------------------------------------------------------------------------

   Usage:  piktd [-GhT]

           -G              run in debug mode
           -h              show program help
           -T              run in test mode

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <time.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <sys/wait.h>

#include "common.h"

const char piktd_version[] = "PIKT piktd version " VERSION;

/* command-line options */
int ischeck = FALSE;
int isdebug = FALSE;
int isdoublecheck = FALSE;
int ishelp = FALSE;
int istest = FALSE;

int uid = 0;		/* default to root */
char *master;
char *domain;
char *accessmode;

char hostname[MAXIDENT+1];
char domainname[MAXIDENT+1];

int lineno = 0;

alt *alerts = (alt *) NULL;
alt *alert;

char *mailcmd, *lpcmd;

unsigned long services = ALL_SERVICES;	/* all services enabled by default */

alt *readconf(void);
int daemon_init(void);

int
main (int argc, char *argv[])
{
	struct tm *tm;
	alt *altp;
	pid_t pid;

	init(argc, argv, (void (*)(void))piktd_usage);

	/* apply the lock, to prevent other concurrent piktd sessions */
	if (islock(piktdlock)) {
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "%s found, another piktd may be active, process aborted", piktdlock);
	}

	daemon_init();

	if (! readconf())
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in main(), no alerts enabled, or error in %s", piktdconf);

	logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "daemon (re)start");
	timingsync();	/* determine secs to next minute */

	while (1) {

		timingsleep();	/* sleep to resync with next minute */

		tm = localtime(&nextmin);	/* get current time */
		nextmin += 60;

		/* set current time bits */
		if (tm->tm_min < 32) {
			lomincurr = (1 << tm->tm_min);
			himincurr = 0L;
		} else {
			lomincurr = 0L;
			himincurr = (1 << (tm->tm_min - 32));
		}
		hrcurr    = (1 << tm->tm_hour);
		domcurr   = (1 << tm->tm_mday);
		moncurr   = (1 << (tm->tm_mon + 1));
		dowcurr   = (1 << tm->tm_wday);

		/* for each alert, look for timing matches */
		foreach (altp, alerts) {
			if (((altp->lomin & lomincurr) || (altp->himin & himincurr)) &&
			    (altp->hr  & hrcurr)                                  &&
			    (altp->dom & domcurr)                                 &&
			    (altp->mon & moncurr)                                 &&
			    (altp->dow & dowcurr)
			   ) {
				/* we found a match, launch the alert */
				switch (pid = fork()) {
					case -1:	/* error */
						logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE,  "in main(), fork error");
						break;
					case 0:		/* child */
						logreturn(PIKTLOG, INFO, __FILE__, __LINE__, FALSE, "%s", altp->piktcmd);
						execl(SHELL, "sh", "-c", altp->piktcmd, (char *) NULL);
						_exit(127);
						break;
					default:	/* parent */
						#if defined(TIME_TO_SLEEP)
						while (!waitpid(pid, NULL, WNOHANG))
							sleep(TIME_TO_SLEEP);
						#else
						waitpid(pid, NULL, WNOHANG);
						#endif
						break;
				}
			}
		}
	}

}

/* output program usage to screen
 */
void
piktd_usage (void)
{
	printf("\nUsage:  piktd [-GhT]\n\n");
	printf("\t-G\t\trun in debug mode\n");
	printf("\t-h\t\tdisplay program help\n");
	printf("\t-T\t\trun in test mode\n\n");
	return;
}

/* read in the piktd.conf file; for each alert, determine the timing, and
   the indicated pikt command string
 */
alt *
readconf (void)
{
	FILE *conffile, *altfile;
	char line[MAXLINE+1];
	char minutes[MAXLINE+1], hours[MAXLINE+1], days_of_month[MAXLINE+1], months[MAXLINE+1], days_of_week[MAXLINE+1], cmd[MAXLINE+1];
	char *cp;
	char c;
	alt *naltp;

	altfile = (FILE *) NULL;
	if ((conffile = fopen(piktdconf, "r")) == NULL)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in readconf(), fopen() error for %s", piktdconf);
	line[MAXLINE-1] = '\0';
	while (fgets(line, MAXLINE+1, conffile) != NULL) {
		if (((c = line[MAXLINE-1]) != '\0') && (c != '\n'))
			logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in readconf(), in %s, line too long: %s", piktdconf, line);
		if ((cp = strchr(line, '\n')) != NULL)
			*cp = '\0';
		if (line[0] == '#')
			continue;
		if ((sscanf(line, "%s %s %s %s %s %[^&]", minutes, hours, days_of_month, months, days_of_week, cmd) != 6) ||
		    ((cp = strstr(cmd, "+A ")) == NULL) ||
		    (cp && !isalpha((int)*(cp+3)))            ) {
			logreturn(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in readconf(), bad %s line: %s", piktdconf, line);
		} else {
			if (!altfile) {
				if ((altfile = fopen(piktdalt, "w")) == NULL)
					logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in readconf, fopen() error for %s", piktdalt);
			}
			fprintf(altfile, "%s\n", cp+3);
			naltp = (alt *) malloc(sizeof(alt));
			parsetiming(minutes,     &(naltp->himin), 32, 59,  0, 59);
			parsetiming(minutes,     &(naltp->lomin),  0, 31,  0, 59);
			parsetiming(hours,          &(naltp->hr),  0, 23,  0, 23);
			parsetiming(days_of_month, &(naltp->dom),  1, 31,  1, 31);
			parsetiming(months,        &(naltp->mon),  1, 12,  1, 12);
			parsetiming(days_of_week,  &(naltp->dow),  0,  6,  0,  6);
			naltp->piktcmd = strdup(cmd);
			naltp->next = (alt *) NULL;
			if (! alerts)
				alerts = naltp;
			else
				alert->next = naltp;
			alert = naltp;
		}
		line[MAXLINE-1] = '\0';
	}
	if (altfile) {
		if (fclose(altfile))
			logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE,  "in readconf(), fclose() error for %s", piktdalt);
	}
	chmod(piktdalt, (mode_t)(S_IRUSR|S_IWUSR|S_IRGRP));	/* 640 */
	if (fclose(conffile))
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, TRUE, "in readconf(), fclose() error for %s", piktdconf);
	return alerts;
}

/* set up in daemon mode
 */
int
daemon_init (void)
{
	pid_t pid;
	static char pidstr[8];

	if ((pid = fork()) < 0)
		return(-1);
	else if (pid != 0)
		exit(0);	/* parent goes bye-bye */

	/* child continues */

	pid = getpid();
	sprintf(pidstr, "%u", (unsigned)pid);
	if (mklock(piktdlock, pidstr, FALSE, TRUE) == ERR)
		logexit(PIKTLOG, ERR, __FILE__, __LINE__, FALSE, "in daemon_init(), mklock() error for %s", piktdlock);

	setsid();		/* become session leader */

	chdir("/");		/* change working directory */

	umask(0);		/* clear our file mode creation mask */

	return(0);
}
